/*
  Copyright (C) 2005, 2006 Sebastien Bourdeauducq <lekernel@prism54.org>
 
  This file is part of p54u.
 
  p54u is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  p54u is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with p54u ; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __EEPROM_H
#define __EEPROM_H

#include <sys/types.h>

#define P54U_EEPROM_BLOCK         0x03fc
#define P54U_EEPROM_SIZE          0x2000

#define P54U_EEPROM_SIGNATURE     0xaa554d47

/*
   In most of the chips, the PDA is in an EEPROM. There's a small header before the
   PDA proper, which is used to store init data for the ROM.
*/
struct p54u_eeprom_header
{
	uint32_t  signature;
	uint16_t  pad;
	/* If firmware_length is not zero, the EEPROM contains a firmware */
	uint16_t  firmware_length; /* in bytes */
	uint32_t  firmware[];
	/* the firmware must be terminated by its CRC32 */
	/* if firmware_length is zero, the CRC should be present anyway and equal to 0xffffffff */
	/* the PDA starts right after this CRC */
} __attribute__ ((packed));

struct p54u_softc;

int p54u_eeprom_parse(struct p54u_softc *sc, uint8_t *eeprom_data, unsigned int eeprom_size);
int p54u_eeprom_read(struct p54u_softc *sc);

#endif /* __EEPROM_H */
