/*
  Copyright (C) 2005, 2006 Sebastien Bourdeauducq <lekernel@prism54.org>
  Copyright (C) 2002 Intersil Americas Inc.
 
  This file is part of p54u.
 
  p54u is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  p54u is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with p54u ; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/* firmware.h : BRA parsing, firmware upload */

#ifndef __FIRMWARE_H
#define __FIRMWARE_H

#include "softmac.h"

/***********/
/*** BRA ***/
/***********/

struct p54u_bra_option
{
	uint32_t code;
#define P54U_BRA_CODE_FWTYPE         0x80000001
#define P54U_BRA_CODE_VERSION        0x80000002
#define P54U_BRA_CODE_DEPENDENT_IF   0x80000003
#define P54U_BRA_CODE_EXPOSED_IF     0x80000004
#define P54U_BRA_CODE_DESCR          0x80000101
#define P54U_BRA_CODE_FINAL          0xFF0000FF
	/* length of data, in 32 bit words, le */
	uint32_t length;
	char data[];
}
__attribute__ ((packed));

#define P54U_MAX_VERSION_STRING_SIZE 32
#define P54U_MAX_BUILD_STRING_SIZE   128

#define P54U_FWTYPE_SIZE             4
#define P54U_FWTYPE_SIG_LM86         "LM86"
#define P54U_FWTYPE_SIG_LM87         "LM87"
#define P54U_FWTYPE_SIG_FMAC         "FMAC"

enum p54u_fwtype {
	P54U_FWTYPE_LM86,
	P54U_FWTYPE_LM87,
	P54U_FWTYPE_FMAC
};

/*
 * Interfaces are defined by the following scheme :
 *
 *  -----------------------------------------------------
 * | 16 bits |  16 bits |  16 bits |  16 bits |  16 bits |
 * |  role   |   if ID  |  variant | btm comp | top comp |
 *  -----------------------------------------------------
 */

/* The interface role can be client or server */
#define P54U_BRA_IF_ROLE_SERVER 0x0000
#define P54U_BRA_IF_ROLE_CLIENT 0x8000

/* The interface ID is a unique identification of a specific interface */
#define P54U_BRA_IF_ID_ISL36356A              0x0001  // Interface between BR_COMP_ID_ISL36356A <-> Firmware
//                                            0x0002  Reserved
#define P54U_BRA_IF_ID_MVC                    0x0003  // MAC Virtual Coprocessor
#define P54U_BRA_IF_ID_SNMP_IF                0x0004  // SNMP Interface between Application <-> Firmware
#define P54U_BRA_IF_ID_SNWNMP_IF              0x0005  // SNWNMP Interface between Application <-> Firmware
#define P54U_BRA_IF_ID_HTTP_IF                0x0006  // HTTP Interface between Application <-> Firmware
#define P54U_BRA_IF_ID_MANUF_TST_IF           0x0007  // Manufacturing and Test Interface between Application <-> Firmware
#define P54U_BRA_IF_ID_DEBUG_IF               0x0008  // PolDebug Interface between Application <-> Firmware
// The PRODUCT and OEM interface are conceptual interfaces for product differentiation
#define P54U_BRA_IF_ID_PRODUCT_IF             0x0009
#define P54U_BRA_IF_ID_OEM_IF                 0x000A
#define P54U_BRA_IF_ID_PCIHOST_IF             0x000B  // Host PCI Interface
#define P54U_BRA_IF_ID_PCI3877_IF             P54U_BRA_IF_ID_PCIHOST_IF     // 3877 <-> Host PCI Interface
#define P54U_BRA_IF_ID_DAP1_MAP               P54U_BRA_IF_ID_PCIHOST_IF     // Interface between BR_COMP_ID_DAP1 <-> Micro AP Firmware
#define P54U_BRA_IF_ID_ISL37704C              0x000C  // Interface between BR_COMP_ID_ISL37704C <-> Firmware
#define P54U_BRA_IF_ID_ISL37700M              P54U_BRA_IF_ID_ISL37704C      // Interface between BR_COMP_ID_ISL37700M <-> Firmware
#define P54U_BRA_IF_ID_TFTP                   0x000D  // TFTP Interface Application <-> Firmware
#define P54U_BRA_IF_ID_STFTP                  0x000E  // STFTP Interface between Application <-> Firmware
#define P54U_BRA_IF_ID_ISL39000               0x000F  // Interface between BR_COMP_ID_ISL39000 <-> Firmware
#define P54U_BRA_IF_ID_ISL39000M              P54U_BRA_IF_ID_ISL39000  // Interface between BR_COMP_ID_ISL39000M <-> Firmware
#define P54U_BRA_IF_ID_ISL39300A              0x0010  // Interface between BR_COMP_ID_ISL39300A <-> Firmware
#define P54U_BRA_IF_ID_BOOT_IF                0x0011  // Interface of bootloader and remaining firmware
#define P54U_BRA_IF_ID_DBO1                   P54U_BRA_IF_ID_BOOT_IF  // Interface between BR_COMP_ID_DBO1 <-> Firmware
//                                            0x0012  Reserved
#define P54U_BRA_IF_ID_FS_IF                  0x0013  // File set interface (size, format, etc)
#define P54U_BRA_IF_ID_DAP1_FS                P54U_BRA_IF_ID_FS_IF  // Interface between BR_COMP_ID_DAP1 <-> File Set
//                                            0x0014  Reserved
//                                            0x0015  Reserved
#define P54U_BRA_IF_ID_ISL37700_UAP_IF        0x0016  // Interface between ISL37700 uAP firmware <-> Firmware
#define P54U_BRA_IF_ID_ISL39000_UAP_IF        0x0017  // Interface between ISL39000 uAP firmware <-> Firmware
// The Primary PRODUCT and Primary OEM interface are conceptual interfaces for product differentiation
#define P54U_BRA_IF_ID_PRIMARY_PRODUCT_IF     0x0018
#define P54U_BRA_IF_ID_PRIMARY_OEM_IF         0x0019
#define P54U_BRA_IF_ID_LMAC_IF                0x001A  // Interface exposed by LMAC

/*
 * The dependent interface defines which interfaces a component
 * needs for its proper operation. These can be mached against
 * the exposed interfaces of other components that are available.
 */
struct p54u_bra_dependent_if {
	uint16_t role;
	uint16_t if_id;
	uint16_t variant;
}
__attribute__ ((packed));

/*
 * The exposed interface defines which interfaces a component
 * offers to other components. Bottom and Top compatibility define
 * with which variants this interface is compatible with.
 */
struct p54u_bra_exposed_if {
	uint16_t role;
	uint16_t if_id;
	uint16_t variant;
	uint16_t bottom_compat;
	uint16_t top_compat;
	uint16_t unk1;
	uint16_t unk2;
	uint16_t physical_variant;
	uint16_t physical_bottom_compat;
	uint16_t physical_top_compat;
}
__attribute__ ((packed));

struct p54u_bra_lmac_description {
	uint32_t unk1;
	uint32_t frame_space_start;
	uint32_t frame_space_end;
	uint32_t unk2;
	uint32_t unk3;
	uint8_t rates[P54U_LMAC_MAXRATES];
}
__attribute__ ((packed));

/*****************/
/*** X2 header ***/
/*****************/

#define P54U_X2_SIGNATURE_SIZE     4
#define P54U_X2_SIGNATURE          "x2  "

struct p54u_x2_header
{
	uint8_t signature[P54U_X2_SIGNATURE_SIZE];
	uint32_t startaddr;
	uint32_t size;
	uint32_t crc; /* CRC32 on the "startaddr" and "size" fields only */
}
__attribute__ ((packed));

/********************/
/*** Loading info ***/
/********************/

#define P54U_FIRMWARE_BLOCK               512

#define P54U_FIRMWARE_DMA_ENGINE_BASE     0xc0000f00

/**************************/
/*** Exported functions ***/
/**************************/

struct p54u_softc;

int p54u_parse_bra(struct p54u_softc *sc, const char *data, unsigned int len);
int p54u_upload_firmware(struct p54u_softc *sc, const char *fw, unsigned int fwsize);
int p54u_init_firmware(struct p54u_softc *sc, char *fw, unsigned int size);

#endif /* __FIRMWARE_H */
