/*
  Copyright (C) 2005, 2006 Sebastien Bourdeauducq <lekernel@prism54.org>
 
  This file is part of p54u.
 
  p54u is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  p54u is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with p54u ; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __FREEMAC_H
#define __FREEMAC_H

#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/mbuf.h>

#define P54U_FMAC_HEADROOM            0x0000

#define P54U_FMAC_CONTROL_BUFFER_SIZE 0x0400

struct p54u_fmac_physical_tx {
	uint32_t address;
	uint16_t size;
	uint16_t pad0;
	uint32_t pad2;
	uint32_t pad3;
} __attribute__((packed));

#define P54U_FMAC_TYPE_LEDS           0x0000

struct p54u_fmac_leds {
	uint16_t packet_type;
	uint16_t mode;
	uint16_t perm_setting;
	uint16_t temp_setting;
	uint16_t duration;
} __attribute__((packed));

#define P54U_FMAC_LED_MODE_SET        0x01
#define P54U_FMAC_LED_MODE_RXBLINK    0x02
#define P54U_FMAC_LED_MODE_TXBLINK    0x03
#define P54U_FMAC_LED_MODE_ACTBLINK   0x04
#define P54U_FMAC_LED_MODE_PWM        0x05
#define P54U_FMAC_LED_A               0x01
#define P54U_FMAC_LED_B               0x02

#define P54U_FMAC_TYPE_EEPROM         0x0001

struct p54u_fmac_eeprom {
	uint16_t packet_type;
	uint16_t offset;
	uint16_t length;
	char data[];
} __attribute__((packed));

struct p54u_softc;

#ifdef WITH_USB
int p54u_fmac_usb_transport(struct p54u_softc *sc, struct mbuf *m, uint32_t address);
#endif

struct mbuf *p54u_fmac_abortaddress(struct p54u_softc *sc, uint32_t req_id, uint32_t aborted_address);
struct mbuf *p54u_fmac_leds(struct p54u_softc *sc, uint16_t mode, uint16_t perm_setting, uint16_t temp_setting, uint16_t duration);
struct mbuf *p54u_fmac_eeprom_readreq(struct p54u_softc *sc, uint16_t offset, uint16_t length);
struct mbuf *p54u_fmac_stats_readreq(struct p54u_softc *sc, uint32_t req_id);

void p54u_fmac_input(struct p54u_softc *sc, struct mbuf *m);

void p54u_fmac_init(struct p54u_softc *sc);
void p54u_fmac_free(struct p54u_softc *sc);

#endif /* __FREEMAC_H */
