/*
  Copyright (C) 2005, 2006 Sebastien Bourdeauducq <lekernel@prism54.org>
  Copyright (C) 2005, 2006 Jean Baptiste Note <jean-baptiste.note@m4x.org>
 
  This file is part of FreeMAC and p54u.
 
  FreeMAC and p54u are free software; you can redistribute them and/or modify
  them under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  FreeMAC and p54u are distributed in the hope that they will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with FreeMAC or p54u ; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/* isl38xx_hardware.h : ISL38xx hardware addresses and constants */

#ifndef __ISL38XX_HARDWARE_H
#define __ISL38XX_HARDWARE_H

#ifndef ISLPTR
#define ISLPTR(x) (*((volatile uint32_t *)x))
#endif



/******************************/
/*** Event handling control ***/
/******************************/

#define REG38XX_IRQ_IDENT_MASKED              ISLPTR(0xc0000000)
#define REG38XX_IRQ_IDENT_UNMASKED            ISLPTR(0xc0000004)
#define REG38XX_IRQ_MASK_ENABLE               ISLPTR(0xc0000008)
#define REG38XX_IRQ_MASK_DISABLE              ISLPTR(0xc000000c)

#define REG38XX_FIQ_IDENT_MASKED              ISLPTR(0xc0000100)
#define REG38XX_FIQ_IDENT_UNMASKED            ISLPTR(0xc0000104)
#define REG38XX_FIQ_MASK_ENABLE               ISLPTR(0xc0000108)
#define REG38XX_FIQ_MASK_DISABLE              ISLPTR(0xc000010c)



/**********************/
/*** System control ***/
/**********************/

#define REG38XX_MAC_ID                        ISLPTR(0xc0000200)
/* related to the low-frequency oscillator */
#define REG38XX_UNK204                        ISLPTR(0xc0000204)
#define REG38XX_CORE_THROTTLE                 ISLPTR(0xc0000210)
#define REG38XX_UNK214                        ISLPTR(0xc0000214)

#define ISL38XX_UNK204_INITBBP                4

#define ISL38XX_THROTTLE_NONE                 0
#define ISL38XX_THROTTLE_30000                1
#define ISL38XX_THROTTLE_18000                2
#define ISL38XX_THROTTLE_9000                 3
#define ISL38XX_THROTTLE_4500                 4
#define ISL38XX_THROTTLE_1800                 5
#define ISL38XX_THROTTLE_900                  6
#define ISL38XX_THROTTLE_450                  7



/*************/
/*** Clock ***/
/*************/

#define REG38XX_CLOCK_VALUE                   ISLPTR(0xc0000280)
#define REG38XX_CLOCK_EVENT_CONTROL           ISLPTR(0xc0000288)


#define ISL38XX_CLOCK_EVENT_CONTROL_UNK01     0x01
#define ISL38XX_CLOCK_EVENT_CONTROL_UNK02     0x02
#define ISL38XX_CLOCK_EVENT_CONTROL_RESET     0x04
/* the two following may be unused */
#define ISL38XX_CLOCK_EVENT_CONTROL_UNK08     0x08
#define ISL38XX_CLOCK_EVENT_CONTROL_UNK10     0x10


/* Events triggered by the clock */
#define ISL38XX_EVENT_CLOCK_UNK1              0x000004
#define ISL38XX_EVENT_CLOCK_UNK2              0x000001



/**************/
/*** Timers ***/
/**************/

/* When the timer reaches 0, the value is copied from this register */
#define REG38XX_TIMER1_INITVALUE              ISLPTR(0xc0000300)
#define REG38XX_TIMER1_VALUE                  ISLPTR(0xc0000304)
#define REG38XX_TIMER1_CONTROL                ISLPTR(0xc0000308)
/* When _CONTROL is set to 0, most of the time _UNK is set to 0 too */
#define REG38XX_TIMER1_UNK30C                 ISLPTR(0xc000030c)

#define REG38XX_TIMER2_INITVALUE              ISLPTR(0xc0000310)
#define REG38XX_TIMER2_VALUE                  ISLPTR(0xc0000314)
#define REG38XX_TIMER2_CONTROL                ISLPTR(0xc0000318)
#define REG38XX_TIMER2_UNK31C                 ISLPTR(0xc000031c)

#define REG38XX_TIMER3_INITVALUE              ISLPTR(0xc0000320)
#define REG38XX_TIMER3_VALUE                  ISLPTR(0xc0000324)
#define REG38XX_TIMER3_CONTROL                ISLPTR(0xc0000328)
#define REG38XX_TIMER3_UNK32C                 ISLPTR(0xc000032c)


/* not sure about these */
#define ISL38XX_TIMER_CONTROL_UNK40           0x40
#define ISL38XX_TIMER_CONTROL_COUNTDOWN       0x80


#define ISL38XX_EVENT_TIMER                   0x020000


/************/
/*** GPIO ***/
/************/

/* Those are 16 bits registers : */
/* Set in the most significant bits the bitmask, and in the least significant bits the actual value */
#define REG38XX_GPIO1_WRITE                    ISLPTR(0xc0000380)
#define REG38XX_GPIO1_READ                     ISLPTR(0xc0000384)
#define REG38XX_GPIO1_CONTROL                  ISLPTR(0xc0000388)

#define REG38XX_GPIO2_WRITE                    ISLPTR(0xc0000400)
#define REG38XX_GPIO2_READ                     ISLPTR(0xc0000404)
#define REG38XX_GPIO2_CONTROL                  ISLPTR(0xc0000408)


/* Common GPIO connections */
#define ISL38XX_GPIO1_NET2280                  0x0004
#define ISL38XX_GPIO1_ANTDIV                   0x0010
#define ISL38XX_GPIO2_SCL                      0x0001
#define ISL38XX_GPIO2_SDA                      0x0002
#define ISL38XX_GPIO2_LED_A                    0x4000
#define ISL38XX_GPIO2_LED_B                    0x0200


#define I2C_EEPROM_ADDRESS                     0xa0
#define I2C_READ                               0x01



/*********************************/
/*** Cryptography acceleration ***/
/*********************************/

#define REG38XX_CRYPT_UNK50C                  ISLPTR(0xc000050c)
#define REG38XX_CRYPT_UNK504                  ISLPTR(0xc0000504)
#define REG38XX_CRYPT_OUTPUT                  ISLPTR(0xc0000510)
#define REG38XX_CRYPT_UNK524                  ISLPTR(0xc0000524)
#define REG38XX_CRYPT_KEY1                    ISLPTR(0xc0000530)
#define REG38XX_CRYPT_KEY2                    ISLPTR(0xc0000534)
#define REG38XX_CRYPT_KEY3                    ISLPTR(0xc0000538)
#define REG38XX_CRYPT_KEY4                    ISLPTR(0xc000053c)
#define REG38XX_CRYPT_KEY5                    ISLPTR(0xc0000540)
#define REG38XX_CRYPT_KEY6                    ISLPTR(0xc0000544)
#define REG38XX_CRYPT_KEY7                    ISLPTR(0xc0000548)
#define REG38XX_CRYPT_KEY8                    ISLPTR(0xc000054c)
#define REG38XX_CRYPT_UNK554                  ISLPTR(0xc0000554)


/* The behavior of the status and control registers is the same as for PCI */
#define REG38XX_CRYPT_INDMA_CONTROL           ISLPTR(0xc0000f80)
#define REG38XX_CRYPT_INDMA_MAXCNT            ISLPTR(0xc0000f84)
#define REG38XX_CRYPT_INDMA_BASE              ISLPTR(0xc0000f88)
#define REG38XX_CRYPT_INDMA_STATUS            ISLPTR(0xc0000f8c)
/* f80 is never used, fa0 seems to control both --vda */
#define REG38XX_CRYPT_OUTDMA_CONTROL          ISLPTR(0xc0000fa0)
#define REG38XX_CRYPT_OUTDMA_MAXCNT           ISLPTR(0xc0000fa4)
#define REG38XX_CRYPT_OUTDMA_BASE             ISLPTR(0xc0000fa8)
#define REG38XX_CRYPT_OUTDMA_STATUS           ISLPTR(0xc0000fac)


#define ISL38XX_EVENT_CRYPT_UNK008000         0x008000
#define ISL38XX_EVENT_CRYPT_UNK010000         0x010000



/*************************/
/*** PCI communication ***/
/*************************/

/* 3880 and 3886 only */

#define REG38XX_DEV_INT_REG                   ISLPTR(0xc0000900)
#define REG38XX_DEV_INT_ACK_REG               ISLPTR(0xc0000904)
#define REG38XX_DEV_INT_EN_REG                ISLPTR(0xc0000908)

#define REG38XX_INT_IDENT_REG                 ISLPTR(0xc0000910)
#define REG38XX_INT_ACK_REG                   ISLPTR(0xc0000914)
#define REG38XX_INT_EN_REG                    ISLPTR(0xc0000918)

#define REG38XX_GEN_PURP_COM_REG_1            ISLPTR(0xc0000920)
#define REG38XX_GEN_PURP_COM_REG_2            ISLPTR(0xc0000924)

#define REG38XX_DIR_MEM_BASE_REG              ISLPTR(0xc0000930)

#define REG38XX_CTRL_STAT_REG                 ISLPTR(0xc0000978)

/*************************/
/*** USB communication ***/
/*************************/

/* 3887 only */

/* The USB controller seems very flexible, allowing configuration */
/* of USB strings, ID numbers etc. from these registers */

/* see function after L5260 in 2.5.8 for example use : */
/*
	mov     r1, #256        @ 0x100
	str     r1, [r4, #0x0028] (REG38XX_EP81_PACKETFRAME)
	mov     r1, #256        @ 0x100
	str     r1, [r4, #0x002c] (REG38XX_EP81_MAXPACKETSIZE)
	ldr     r1, D55e0 (0x00027890)
	str     r1, [r4, #0x0034] (REG38XX_EP81_DESCPTR)
	mov     r1, #2
	str     r1, [r4, #0x0020] (REG38XX_EP81_CONTROL)
*/

#define REG38XX_EP81_UNK000                   ISLPTR(0xc1000000)
#define REG38XX_EP81_UNKPTR                   ISLPTR(0xc1000014)
/* 2.5.8:
L4f1c:  (...)
	mov     r2, #64 @ 0x40
	mov     r0, #0xc1000000
	str     r2, [r0, #0x0024] (REG38XX_EP81_STATUS)
	mov     r2, #40 @ 0x28
	str     r2, [r0, #0x0020] (REG38XX_EP81_CONTROL)
*/
#define REG38XX_EP81_CONTROL                  ISLPTR(0xc1000020)
#define REG38XX_EP81_STATUS                   ISLPTR(0xc1000024)
#define REG38XX_EP81_PACKETFRAME              ISLPTR(0xc1000028)
#define REG38XX_EP81_MAXPACKETSIZE            ISLPTR(0xc100002c)
#define REG38XX_EP81_SETUPTR                  ISLPTR(0xc1000030)
#define REG38XX_EP81_DESCPTR                  ISLPTR(0xc1000034)
/* contains an address */
#define REG38XX_EP81_UNK054                   ISLPTR(0xc1000054)

/* EP 0x82 at 0xc10001xx ? */

#define REG38XX_EP01_UNK200                   ISLPTR(0xc1000200)
#define REG38XX_EP01_UNKPTR                   ISLPTR(0xc1000214)
#define REG38XX_EP01_CONTROL                  ISLPTR(0xc1000220)
#define REG38XX_EP01_STATUS                   ISLPTR(0xc1000224)
#define REG38XX_EP01_PACKETFRAME              ISLPTR(0xc1000228)
#define REG38XX_EP01_MAXPACKETSIZE            ISLPTR(0xc100022c)
#define REG38XX_EP01_SETUPTR                  ISLPTR(0xc1000230)
#define REG38XX_EP01_DESCPTR                  ISLPTR(0xc1000234)
#define REG38XX_EP01_UNK254                   ISLPTR(0xc1000254)

/* EP 0x02 at 0xc10003xx ? */

/* not critical */
#define REG38XX_EP_UNK060                     ISLPTR(0xc1000060)
/* not critical */
#define REG38XX_EP_UNK064                     ISLPTR(0xc1000064)

/* critical */
#define REG38XX_EP_UNK404                     ISLPTR(0xc1000404)
/* not critical */
#define REG38XX_EP_UNK408                     ISLPTR(0xc1000408)
/* not critical */
#define REG38XX_EP_UNK40C                     ISLPTR(0xc100040c)
/* not critical */
#define REG38XX_EP_UNK414                     ISLPTR(0xc1000414)

/* "standard" rw register, bit 2 enables/disables interrupts, probably */
#define REG38XX_EP_INT_DISABLE                ISLPTR(0xc1000418)

/* All *PTR registers point to this type of structure */
/* They can be chained, maybe to buffer multiple frames */
struct isl38xx_usb_descriptor
{
	unsigned flags;    /* Not sure what writes to it (ARM or USB controller) */
	unsigned reserved; /* Always zero */
	void *data;
	struct isl38xx_usb_descriptor *next;
};

/* On the 3886 : */
/* This event is triggered when REG38XX_DEV_INT_REG & REG38XX_DEV_INT_EN_REG != 0 */
/* On the 3887 : */
/* This event is triggered when data comes from an endpoint */
#define ISL38XX_EVENT_HOST                    0x000040



/**********************/
/*** PCI DMA engine ***/
/**********************/

/* 3880 and 3886 only */

/* An ARM->PCI DMA transfer is triggered by doing the following:
 * * REG38XX_TXDMA_CONTROL must be equal to 1
 * * write the size of the transfer in bytes to REG38XX_TXDMA_MAXCNT
 * * write the target address (ARM) to REG38XX_TXDMA_BASE
 * * write the source address (PCI) to REG38XX_DMA_MASTER_ADDRESS
 * * write the size of the transfer in 32-bits words to REG38XX_DMA_MASTER_LEN
 * * write 0xc to REG38XX_DMA_MASTER_CONTROL
 *
 * REG38XX_RXDMA_STATUS reads 0 if REG38XX_RXDMA_CONTROL is disabled
 * REG38XX_RXDMA_STATUS goes 0->3 when REG38XX_RXDMA_CONTROL is enabled
 * REG38XX_RXDMA_STATUS goes 3->2 when REG38XX_RXDMA_BASE gets written to
 * REG38XX_RXDMA_STATUS goes 2->3 when transfer is complete
 */

#define REG38XX_TXDMA_CONTROL                 ISLPTR(0xc0000f00)
#define REG38XX_TXDMA_MAXCNT                  ISLPTR(0xc0000f04)
#define REG38XX_TXDMA_BASE                    ISLPTR(0xc0000f08)
#define REG38XX_TXDMA_STATUS                  ISLPTR(0xc0000f0c)
#define REG38XX_TXDMA_CURRENT                 ISLPTR(0xc0000f10)
#define REG38XX_TXDMA_REMAIN                  ISLPTR(0xc0000f14)

/* PCI->ARM DMA transfer: same as ARM->PCI (see above), but
 * write 0x4 instead of 0xc to REG38XX_DMA_MASTER_CONTROL
 */

#define REG38XX_RXDMA_CONTROL                 ISLPTR(0xc0000f20)
#define REG38XX_RXDMA_MAXCNT                  ISLPTR(0xc0000f24)
#define REG38XX_RXDMA_BASE                    ISLPTR(0xc0000f28)
#define REG38XX_RXDMA_STATUS                  ISLPTR(0xc0000f2c)
#define REG38XX_RXDMA_CURRENT                 ISLPTR(0xc0000f30)
#define REG38XX_RXDMA_REMAIN                  ISLPTR(0xc0000f34)

#define REG38XX_DMA_MASTER_ADDRESS            ISLPTR(0xc0000960)
#define REG38XX_DMA_MASTER_LEN                ISLPTR(0xc0000964)
#define REG38XX_DMA_MASTER_CONTROL            ISLPTR(0xc0000968)
/* The behaviour of this register is strange */
/* It reads a memory location in the chip */
/* On my device, it reads at 0x00021f00 by default */
/* and the copy is not updated when we rewrite memory */
/* The effect of writing the register is unknown. It does not write to memory */
/* After a DMA read, it contains the last word transferred */
/* After a DMA write, it contains a bogus word from the ARM memory */
/* which is at 0x00021f00+number of bytes transferred over DMA */
/* maybe the DMA TX bug is related ? */
#define REG38XX_DMA_MASTER_UNK970             ISLPTR(0xc0000970)


#define ISL38XX_DMA_STATUS_DISABLED           0x00000000
#define ISL38XX_DMA_STATUS_DONE               0x00000001
#define ISL38XX_DMA_STATUS_READY              0x00000002
/* seen on DMA TX with UNK10 */
#define ISL38XX_DMA_STATUS_DONE_UNK100        0x00000100

#define ISL38XX_DMA_CONTROL_DISABLE           0x00000000
#define ISL38XX_DMA_CONTROL_ENABLE            0x00000001

#define ISL38XX_DMA_MASTER_CONTROL_TRIGGER    0x00000004
#define ISL38XX_DMA_MASTER_CONTROL_OUT        0x00000008
/* this is not "no event". the event disable must be a side effect. */
/* DMA RX fails when this flag is set */
#define ISL38XX_DMA_MASTER_CONTROL_UNK10      0x00000010


#define ISL38XX_EVENT_TXDMA                   0x000200
#define ISL38XX_EVENT_RXDMA                   0x000400



/***********/
/*** BBP ***/
/***********/

/* format : */
/* 0x00000xxy where xx = main chip type (80, 86, 87, 90) */
/* y = probably silicon revision */
#define REG38XX_BBP_ID                        ISLPTR(0xc0000600)

/* FreeMAC was never tried on Prism Indigo (3877), feedback welcome */
#define ISL38XX_BBP_ID_3877                   0x77
#define ISL38XX_BBP_ID_3880                   0x80
#define ISL38XX_BBP_ID_3886                   0x86
#define ISL38XX_BBP_ID_3887                   0x87
#define ISL38XX_BBP_ID_3890                   0x90

#define REG38XX_BBP_TXDMA_CONTROL             ISLPTR(0xc0000f40)
#define REG38XX_BBP_TXDMA_MAXCNT              ISLPTR(0xc0000f44)
#define REG38XX_BBP_TXDMA_BASE                ISLPTR(0xc0000f48)
#define REG38XX_BBP_TXDMA_STATUS              ISLPTR(0xc0000f4c)
//#define REG38XX_BBP_TXDMA_CURRENT             ISLPTR(0xc0000f50)
//#define REG38XX_BBP_TXDMA_REMAIN              ISLPTR(0xc0000f54)

#define REG38XX_BBP_RXDMA_CONTROL             ISLPTR(0xc0000f60)
#define REG38XX_BBP_RXDMA_MAXCNT              ISLPTR(0xc0000f64)
#define REG38XX_BBP_RXDMA_BASE                ISLPTR(0xc0000f68)
#define REG38XX_BBP_RXDMA_STATUS              ISLPTR(0xc0000f6c)
//#define REG38XX_BBP_RXDMA_CURRENT             ISLPTR(0xc0000f70)
//#define REG38XX_BBP_RXDMA_REMAIN              ISLPTR(0xc0000f74)



/***********/
/*** ROM ***/
/***********/

#define ISL38XX_ROM_BASE                      0xe0000000
/* The debugger is passed the following information : */
/* r2 = previous state of CPSR (we must disable IRQs and FIQs before entering debugger) */
/* r4 = 32-bit magic describing the reason for entering debugger */
/* r7 = 0x0003ff20 = ISL38XX_DEBUG_R7 ; perhaps the debug info area can be put somewhere else */
/* than at 0x0003ffxx. Should check. */
#define ISL38XX_ROM_DEBUGGER_3886              0xe0000034
#define ISL38XX_ROM_DEBUGGER_3887              0xe0000040

/* in 0x0003xxxx range for compatibility with FullMAC chip */
#define ISL38XX_DEBUG_SVC_R0                  ISLPTR(0x0003ff00)
#define ISL38XX_DEBUG_SVC_R1                  ISLPTR(0x0003ff04)
#define ISL38XX_DEBUG_SVC_R2                  ISLPTR(0x0003ff08)
#define ISL38XX_DEBUG_SVC_R3                  ISLPTR(0x0003ff0c)
#define ISL38XX_DEBUG_SVC_R4                  ISLPTR(0x0003ff10)
#define ISL38XX_DEBUG_SVC_R5                  ISLPTR(0x0003ff14)
#define ISL38XX_DEBUG_SVC_R6                  ISLPTR(0x0003ff18)
#define ISL38XX_DEBUG_SVC_R7                  ISLPTR(0x0003ff1c)
#define ISL38XX_DEBUG_SVC_R8                  ISLPTR(0x0003ff20)
#define ISL38XX_DEBUG_SVC_R9                  ISLPTR(0x0003ff24)
#define ISL38XX_DEBUG_SVC_SL                  ISLPTR(0x0003ff28)
#define ISL38XX_DEBUG_SVC_FP                  ISLPTR(0x0003ff2c)
#define ISL38XX_DEBUG_SVC_IP                  ISLPTR(0x0003ff30)
#define ISL38XX_DEBUG_SVC_SP                  ISLPTR(0x0003ff34)
#define ISL38XX_DEBUG_SVC_LR                  ISLPTR(0x0003ff38)
#define ISL38XX_DEBUG_SVC_PC                  ISLPTR(0x0003ff3c)
#define ISL38XX_DEBUG_SVC_SS                  ISLPTR(0x0003ff40)
#define ISL38XX_DEBUG_SVC_CS                  ISLPTR(0x0003ff44)

#define ISL38XX_DEBUG_USR_SP                  ISLPTR(0x0003ff48)
#define ISL38XX_DEBUG_USR_LR                  ISLPTR(0x0003ff4c)

#define ISL38XX_DEBUG_ABT_SP                  ISLPTR(0x0003ff50)
#define ISL38XX_DEBUG_ABT_LR                  ISLPTR(0x0003ff54)
#define ISL38XX_DEBUG_ABT_SS                  ISLPTR(0x0003ff58)

#define ISL38XX_DEBUG_UND_SP                  ISLPTR(0x0003ff5c)
#define ISL38XX_DEBUG_UND_LR                  ISLPTR(0x0003ff60)
#define ISL38XX_DEBUG_UND_SS                  ISLPTR(0x0003ff64)

#define ISL38XX_DEBUG_IRQ_SP                  ISLPTR(0x0003ff68)
#define ISL38XX_DEBUG_IRQ_LR                  ISLPTR(0x0003ff6c)
#define ISL38XX_DEBUG_IRQ_SS                  ISLPTR(0x0003ff70)

#define ISL38XX_DEBUG_FIQ_R8                  ISLPTR(0x0003ff74)
#define ISL38XX_DEBUG_FIQ_R9                  ISLPTR(0x0003ff78)
#define ISL38XX_DEBUG_FIQ_F1                  ISLPTR(0x0003ff7c)
#define ISL38XX_DEBUG_FIQ_F2                  ISLPTR(0x0003ff80)
#define ISL38XX_DEBUG_FIQ_F3                  ISLPTR(0x0003ff84)
#define ISL38XX_DEBUG_FIQ_SP                  ISLPTR(0x0003ff88)
#define ISL38XX_DEBUG_FIQ_LR                  ISLPTR(0x0003ff8c)
#define ISL38XX_DEBUG_FIQ_SS                  ISLPTR(0x0003ff90)

#define ISL38XX_DEBUG_TIME1                   ISLPTR(0x0003ffe8)
#define ISL38XX_DEBUG_TIME2                   ISLPTR(0x0003ffd8)



/*********************/
/*** Unknown stuff ***/
/*********************/

/* those are set on startup, the LMAC handles them as FIQs. Triggered by the BBP when */
/* data arrives or when it needs data to transmit ? */
#define ISL38XX_EVENT_UNK400000               0x400000
#define ISL38XX_EVENT_UNK000008               0x000008

/* the LMAC handles those as IRQs */
#define ISL38XX_EVENT_UNK000002               0x000002
#define ISL38XX_EVENT_UNK000100               0x000100 /* handled as FIQ by the 3887 BootROM */
#define ISL38XX_EVENT_UNK001000               0x001000
#define ISL38XX_EVENT_UNK080000               0x080000

#define REG38XX_UNK800                        ISLPTR(0xc0000800)

#endif /* __ISL38XX_HARDWARE_H */
