/*
  Copyright (C) 2005, 2006 Sebastien Bourdeauducq <lekernel@prism54.org>
  Copyright (C) 2002 Intersil Americas Inc.
 
  This file is part of p54u.
 
  p54u is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  p54u is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with p54u ; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __ISL38XX_HOST_H
#define __ISL38XX_HOST_H

#define ISL3887_ROM_SIGNATURE_SIZE              4
#define ISL3887_ROM_SIGNATURE                   "!MOR"

/**********************************/
/*** Host Interface Definitions ***/
/**********************************/

#define ISL38XX_PCI_MEM_SIZE                    0x2000
#define ISL38XX_MEMORY_WINDOW_SIZE              0x1000
#define ISL38XX_WRITEIO_DELAY                   10      /* in us */
#define ISL38XX_RESET_DELAY                     50      /* in ms */
#define ISL38XX_WAIT_CYCLE                      100     /* in ms */
#define ISL38XX_MAX_WAIT_CYCLES                 10

/***********************/
/*** PCI Memory Area ***/
/***********************/

#define ISL38XX_HARDWARE_REG                    0x0000
#define ISL38XX_CARDBUS_CIS                     0x0800
#define ISL38XX_DIRECT_MEM_WIN                  0x1000

/**********************/
/* Hardware registers */
/**********************/

#define ISL38XX_DEV_INT_REG                     0x0000
#define ISL38XX_INT_IDENT_REG                   0x0010
#define ISL38XX_INT_ACK_REG                     0x0014
#define ISL38XX_INT_EN_REG                      0x0018
#define ISL38XX_GEN_PURP_COM_REG_1              0x0020
#define ISL38XX_GEN_PURP_COM_REG_2              0x0024
#define ISL38XX_DIR_MEM_BASE_REG                0x0030
#define ISL38XX_DMA_MASTER_ADDRESS              0x0060
#define ISL38XX_DMA_MASTER_LEN                  0x0064
#define ISL38XX_DMA_MASTER_CONTROL              0x0068
#define ISL38XX_CTRL_STAT_REG                   0x0078


/********************************/
/* Control/Status register bits */
/********************************/

#define ISL38XX_CTRL_STAT_SLEEPMODE             0x00000200
#define ISL38XX_CTRL_STAT_CLKRUN                0x00800000
#define ISL38XX_CTRL_STAT_RESET                 0x10000000
#define ISL38XX_CTRL_STAT_RAMBOOT               0x20000000
#define ISL38XX_CTRL_STAT_STARTHALTED           0x40000000
#define ISL38XX_CTRL_STAT_HOST_OVERRIDE         0x80000000

/*********************/
/* PCI DMA transfers */
/*********************/

struct isl38xx_fragment {
	uint32_t address; /* physical address on host */
	uint32_t arm_address; /* address on device */
	uint16_t size;
	uint16_t flags;
} __attribute__((packed));

#define ISL38XX_CB_QCOUNT             4
#define ISL38XX_CB_RX_DATA            0
#define ISL38XX_CB_TX_DATA            1
#define ISL38XX_CB_RX_DATAHIPRI       2
#define ISL38XX_CB_TX_DATAHIPRI       3

#define ISL38XX_CB_RX_DATA_QSIZE      8
#define ISL38XX_CB_TX_DATA_QSIZE      32
#define ISL38XX_CB_RX_DATAHIPRI_QSIZE 4
#define ISL38XX_CB_TX_DATAHIPRI_QSIZE 4
#define ISL38XX_MIN_QTHRESHOLD        4 /* fragments */

struct isl38xx_cb {
	uint32_t driver_curr_frag[ISL38XX_CB_QCOUNT];
	uint32_t device_curr_frag[ISL38XX_CB_QCOUNT];
	struct isl38xx_fragment rx[ISL38XX_CB_RX_DATA_QSIZE];
	struct isl38xx_fragment tx[ISL38XX_CB_TX_DATA_QSIZE];
	struct isl38xx_fragment rx_hipri[ISL38XX_CB_RX_DATAHIPRI_QSIZE];
	struct isl38xx_fragment tx_hipri[ISL38XX_CB_TX_DATAHIPRI_QSIZE];
} __attribute__((packed));

#endif /* __ISL38XX_HOST_H */

