/*
  Copyright (C) 2005, 2006 Sebastien Bourdeauducq <lekernel@prism54.org>
 
  This file is part of p54u.
 
  p54u is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  p54u is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with p54u ; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/* pmalloc.h : Prism Memory ALLOCator */

#ifndef __PMALLOC_H
#define __PMALLOC_H

#include <sys/types.h>
#include <sys/param.h>
#include <sys/lock.h>
#include <sys/mutex.h>

#define P54U_PMALLOC_ALIGNMENT   4
#define P54U_PMALLOC_MAX_BLOCKS  8
#define P54U_PMALLOC_FAILURE     0x00000000

enum p54u_pmalloc_purposes {
	P54U_PMALLOC_PURPOSE_CONTROL,
	P54U_PMALLOC_PURPOSE_EEPROM,
	P54U_PMALLOC_PURPOSE_80211MGMT,
	P54U_PMALLOC_PURPOSE_80211DATA,
	P54U_PMALLOC_PURPOSE_80211BEACON,
	P54U_PMALLOC_PURPOSE_FWRESERVED,
	P54U_PMALLOC_PURPOSE_MISC
};

struct p54u_pmalloc_block
{
	uint32_t start_address;
	uint32_t end_address;
	uint32_t headroom;
	int purpose;
};

struct p54u_pmalloc_state
{
	struct mtx lock;
uint32_t start_address;
uint32_t end_address;
unsigned int allocated_blocks; /* number of allocated blocks */
struct p54u_pmalloc_block blocks[P54U_PMALLOC_MAX_BLOCKS]
				; /* information about the allocated blocks, ordered by start address */
};

uint32_t p54u_pmalloc(struct p54u_pmalloc_state *state, uint32_t size, uint32_t headroom, int purpose);
int p54u_pmfree(struct p54u_pmalloc_state *state, uint32_t address);
uint32_t p54u_pmfind(struct p54u_pmalloc_state *state, int purpose);

void p54u_pmstate_init(struct p54u_pmalloc_state *state, uint32_t start_address, uint32_t end_address);
void p54u_pmstate_free(struct p54u_pmalloc_state *state);

#endif /* __PMALLOC_H */
