/*
  Copyright (C) 2005, 2006 Sebastien Bourdeauducq <lekernel@prism54.org>
  Copyright (C) 2004 Feyd
  Copyright (C) 2004, 2005 Jean-Baptiste Note <jean-baptiste.note@m4x.org>
 
  This file is part of p54u.
 
  p54u is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  p54u is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with p54u ; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __TRANSPORT_H
#define __TRANSPORT_H

#include <sys/types.h>
#include <sys/module.h>
#include <sys/bus.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/mbuf.h>

#ifdef WITH_USB

#include <dev/usb/usb.h>
#include <dev/usb/usbdi.h>

struct p54u_pipe {
	uint8_t addr;                  /* address of the endpoint */

	usbd_pipe_handle handle;
	usbd_xfer_handle xfer;

	void *buf;                     /* the buffer (DMA capable) */
	uint32_t size;                 /* size of the buffer */
};

#define P54U_BRG_POWER_UP   0x00000001 /* GPIO0_DATA */
#define P54U_BRG_POWER_DOWN 0x00000002 /* GPIO1_DATA */

#define NET2280_BASE_REGISTERS    0x10000000
#define NET2280_BASE_ENDPOINTS    0x20000000

#define P54U_TRDY_TIMEOUT     0x40
#define P54U_RETRY_TIMEOUTG   0x41

/* What to send to the NET2280 to read/write registers */
struct p54u_reg {
	uint16_t port;
	uint32_t addr;
	uint32_t val;
} __attribute__ ((packed));

#define P54U_REG_WSIZE    (sizeof(struct p54u_reg))
#define P54U_REG_RSIZE    (P54U_REG_WSIZE - (sizeof(uint32_t)))

/* NET2280 addresses and constants */

#define NET2280_DEVINIT       0x00
#define NET2280_USBIRQENB1    0x24
#define NET2280_IRQSTAT1      0x2c
#define NET2280_FIFOCTL       0x38
#define NET2280_GPIOCTL       0x50
#define NET2280_RELNUM        0x88

#define NET2280_EPA_CFG       0x320
#define NET2280_EPA_RSP       0x324
#define NET2280_EPA_IRQENB    0x328
#define NET2280_EPA_STAT      0x32c
#define NET2280_EPA_AVAIL     0x330
#define NET2280_EPA_DATA      0x334

#define NET2280_EPB_CFG       0x340
#define NET2280_EPB_RSP       0x344
#define NET2280_EPB_IRQENB    0x348
#define NET2280_EPB_STAT      0x34c
#define NET2280_EPB_AVAIL     0x350
#define NET2280_EPB_DATA      0x354

#define NET2280_EPC_CFG       0x360
#define NET2280_EPC_RSP       0x364
#define NET2280_EPC_IRQENB    0x368
#define NET2280_EPC_STAT      0x36c
#define NET2280_EPC_AVAIL     0x370
#define NET2280_EPC_DATA      0x374

#define NET2280_EPD_CFG       0x380
#define NET2280_EPD_RSP       0x384
#define NET2280_EPD_IRQENB    0x388
#define NET2280_EPD_STAT      0x38c
#define NET2280_EPD_AVAIL     0x390
#define NET2280_EPD_DATA      0x394

#define NET2280_USB_INTERRUPT_ENABLE      0x80000000
#define NET2280_PCI_INTA_INTERRUPT        0x01000000
#define NET2280_PCI_INTA_INTERRUPT_ENABLE 0x01000000

#define NET2280_LOCAL_CLOCK_FREQUENCY 8
#define NET2280_CLK_4MHZ       (15 << NET2280_LOCAL_CLOCK_FREQUENCY)
#define NET2280_CLK_30MHZ      (2 << NET2280_LOCAL_CLOCK_FREQUENCY)
#define NET2280_CLK_60MHZ      (1 << NET2280_LOCAL_CLOCK_FREQUENCY)
#define NET2280_CLK_STOP       (0 << NET2280_LOCAL_CLOCK_FREQUENCY)

#define NET2280_PCI_ENABLE     0x00000020
#define NET2280_PCI_SOFT_RESET 0x00000004

#define NET2280_CLEAR_NAK_OUT_PACKETS_MODE 0x00000004
#define NET2280_FIFO_FLUSH                 0x00000200

#endif /* WITH_USB */

#define P54U_MAX_DATA_FRAME_SIZE  2048
#define P54U_MAX_DEV_FRAME_SIZE   32
#define P54U_MAX_BRG_FRAME_SIZE   32
#define P54U_MAX_INT_FRAME_SIZE   4

#define P54U_DEV_BASE   0x40000000

#define P54U_PORT_BRG      0x0010
#define P54U_PORT_BRG_CFG  0x0000
#define P54U_PORT_DEV      0x0800
#define P54U_PORT_DEV_CFG  0x0880
#define P54U_PORT_U16      0x0003
#define P54U_PORT_U32      0x000f

#define P54U_PORT_BRG_U32      (P54U_PORT_BRG | P54U_PORT_U32)
#define P54U_PORT_BRG_CFG_U32  (P54U_PORT_BRG_CFG | P54U_PORT_U32)
#define P54U_PORT_BRG_CFG_U16  (P54U_PORT_BRG_CFG | P54U_PORT_U16)

#define P54U_PORT_DEV_U32      (P54U_PORT_DEV | P54U_PORT_U32)
#define P54U_PORT_DEV_CFG_U32  (P54U_PORT_DEV_CFG | P54U_PORT_U32)
#define P54U_PORT_DEV_CFG_U16  (P54U_PORT_DEV_CFG | P54U_PORT_U16)

/* NET2280 setup : */
/* EPA is endpoint 0x01 (host->device data) */
/* EPB is endpoint 0x81 (device->host data) */
/* EPC is endpoint 0x02 (host->device MGMT - high priority data) */
/* EPD is endpoint 0x82 (device->host MGMT - high priority data) */
/* pipes 3 and 4 are not used by the driver */
/* neither is MGMT pipe with 2.5.x and newer firmwares */

enum p54u_pipe_addr {
	P54U_PIPE_DATA =  0x01,
	P54U_PIPE_MGMT =  0x02,
	P54U_PIPE_3 =     0x03,
	P54U_PIPE_4 =     0x04,
	P54U_PIPE_BRG =   0x0d,
	P54U_PIPE_DEV =   0x0e,
	P54U_PIPE_INT =   0x0f
};

/* Driver parameters */
#define P54U_TRANSFER_TIMEOUT 1000
#define P54U_TRANSFER_TIMEOUT_HZ ((P54U_TRANSFER_TIMEOUT*hz)/1000)

struct p54u_queue;
struct p54u_softc;

#ifdef WITH_USB
int p54u_alloc_pipe(struct p54u_softc *sc, struct p54u_pipe *pipe, uint8_t address, int size);
int p54u_free_pipe(struct p54u_softc *sc, struct p54u_pipe *pipe);
int p54u_alloc_standard_pipes(struct p54u_softc *sc);
int p54u_free_standard_pipes(struct p54u_softc *sc);
int p54u_int_rx_submit(struct p54u_softc *sc);
int p54u_data_rx_submit(struct p54u_softc *sc);
usbd_status p54u_bulk_transfer(struct p54u_softc *sc, struct p54u_pipe *pipe, uint32_t *size, uint16_t flags);
#endif

uint32_t p54u_reg_rw(struct p54u_softc *sc, int write, int ep, int port, uint32_t addr, uint32_t val);
#define p54u_brg_writel(dev, addr, val)          p54u_reg_rw(dev, 1, P54U_PIPE_BRG, P54U_PORT_BRG_U32, addr, val)
#define p54u_pcicfg_brg_writel(dev, addr, val)   p54u_reg_rw(dev, 1, P54U_PIPE_BRG, P54U_PORT_BRG_CFG_U32, addr, val)
#define p54u_pcicfg_brg_writew(dev, addr, val)   p54u_reg_rw(dev, 1, P54U_PIPE_BRG, P54U_PORT_BRG_CFG_U16, addr, ((val)&0xffff))

#define p54u_dev_writel(dev, reg, val)           p54u_reg_rw(dev, 1, P54U_PIPE_DEV, P54U_PORT_DEV_U32, (P54U_DEV_BASE|(reg)), val)
#define p54u_pcicfg_dev_writel(dev, addr, val)   p54u_reg_rw(dev, 1, P54U_PIPE_DEV, P54U_PORT_DEV_CFG_U32, addr, val)
#define p54u_pcicfg_dev_writew(dev, addr, val)   p54u_reg_rw(dev, 1, P54U_PIPE_DEV, P54U_PORT_DEV_CFG_U16, addr, ((val)&0xffff))

#define p54u_brg_readl(dev, addr)                p54u_reg_rw(dev, 0, P54U_PIPE_BRG, P54U_PORT_BRG_U32, addr, 0)
#define p54u_pcicfg_brg_readl(dev, addr)         p54u_reg_rw(dev, 0, P54U_PIPE_BRG, P54U_PORT_BRG_CFG_U32, addr, 0)
#define p54u_pcicfg_brg_readw(dev, addr)        (p54u_reg_rw(dev, 0, P54U_PIPE_BRG, P54U_PORT_BRG_CFG_U16, addr, 0)&0xffff)

#define p54u_dev_readl(dev, reg)                 p54u_reg_rw(dev, 0, P54U_PIPE_DEV, P54U_PORT_DEV_U32, (P54U_DEV_BASE|(reg)), 0)
#define p54u_pcicfg_dev_readl(dev, addr)         p54u_reg_rw(dev, 0, P54U_PIPE_DEV, P54U_PORT_DEV_CFG_U32, addr, 0)
#define p54u_pcicfg_dev_readw(dev, addr)        (p54u_reg_rw(dev, 0, P54U_PIPE_DEV, P54U_PORT_DEV_CFG_U16, addr, 0)&0xffff)

#ifdef WITH_PCI
int p54u_queue_can_submit(struct p54u_softc *sc, struct p54u_queue *q);
int p54u_queue_submit(struct p54u_softc *sc, struct p54u_queue *q, struct mbuf *m, uint32_t arm_address);
int p54u_alloc_queues(struct p54u_softc *sc);
void p54u_free_queues(struct p54u_softc *sc);
#endif

int p54u_initialize_device(struct p54u_softc *sc);

int p54u_uart_start(struct p54u_softc *sc);

int p54u_irq_init(struct p54u_softc *sc);
int p54u_irq_free(struct p54u_softc *sc);

int p54u_devnode_onread(struct p54u_softc *sc);
int p54u_devnode_onwrite(struct p54u_softc *sc);

#ifdef WITH_USB
int p54u_usb_transport(struct p54u_softc *sc, struct mbuf *m);
#endif

#endif /* __TRANSPORT_H */
