/*
  Copyright 2005, 2006 Sebastien Bourdeauducq <lekernel@prism54.org>
 
  This file is part of p54u.
 
  p54u is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  p54u is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with p54u ; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef __P54_IOCTL_H
#define __P54_IOCTL_H

#define P54RESET        _IO  ('P', 10)
#define P54FIRMWARE     _IOW ('P', 11, struct p54_firmware)

#define P54HWINFO       _IOR ('P', 12, struct p54_hwinfo)
#define P54FWINFO       _IOR ('P', 13, struct p54_fwinfo)
#define P54EEPROMDUMP   _IOW ('P', 14, char *)

#define P54STATS        _IOR ('P', 15, struct p54_stats)
#define P54LEDS         _IOR ('P', 16, struct p54_leds)

#define P54PMALLOC      _IOR ('P', 17, struct p54_pmalloc)
#define P54FRAME        _IOW ('P', 18, struct p54_frame)
#define P54ABORTADDR    _IOW ('P', 19, uint32_t)
#define P54MEMREAD      _IOW ('P', 20, struct p54_memread)

#define P54CRASHDUMP    _IOWR('P', 21, struct p54_crashdump)

/*******************/
/*** P54FIRMWARE ***/
/*******************/

struct p54_firmware {
	uint32_t size;
	char *data;
};

/*****************/
/*** P54HWINFO ***/
/*****************/

enum {
	P54_DEVTYPE_USB1 = 0,
	P54_DEVTYPE_USB2 = 1,
	P54_DEVTYPE_PCI = 2,
};

#define P54_PLATFORM_ID_3880 0x55018003
#define P54_PLATFORM_ID_3886 0x55018603
#define P54_PLATFORM_ID_3887 0x55018703

/* keep in sync with P54U_MAX_ID_SIZE (p54u.h) */
#define P54_MAX_ID_SIZE 17

struct p54_hwinfo {
	uint8_t eeprom_available;
	uint8_t device_type;
	/* The following are only meaningful if eeprom_available = 1 */
	char manufacturing_part_number[P54_MAX_ID_SIZE];
	char nic_serial_number[P54_MAX_ID_SIZE];
	uint32_t platform_id;
	uint16_t antenna1_gain;
	uint16_t antenna2_gain;
};

/*****************/
/*** P54FWINFO ***/
/*****************/

/* keep in sync with P54U_MAX_VERSION_STRING_SIZE, P54U_MAX_BUILD_STRING_SIZE and P54U_FWTYPE_SIZE (p54u.h) */
#define P54_MAX_VERSION_STRING_SIZE 32
#define P54_MAX_BUILD_STRING_SIZE   128
#define P54_FWTYPE_SIZE             4

struct p54_fwinfo {
	char version[P54_MAX_VERSION_STRING_SIZE];
	char build[P54_MAX_BUILD_STRING_SIZE];
	char type[P54_FWTYPE_SIZE];
	uint16_t variant;
	uint16_t bottom_compat;
	uint16_t top_compat;
	uint16_t physical_variant;
	uint16_t physical_bottom_compat;
	uint16_t physical_top_compat;
};

/*********************/
/*** P54EEPROMDUMP ***/
/*********************/

/* keep in sync with P54U_EEPROM_SIZE (eeprom.h) */
#define P54_EEPROM_SIZE 0x2000

/****************/
/*** P54STATS ***/
/****************/

struct p54_stats {
	uint32_t rx_packets;
	uint32_t time;
	uint32_t unk1;
	uint32_t unk2;
	uint32_t unk3;
	uint32_t unk4;
	uint32_t unk5;
	uint32_t unk6;
	uint32_t unk7;
};

/***************/
/*** P54LEDS ***/
/***************/

#define P54_LED_MODE_SET        0x01
#define P54_LED_MODE_RXBLINK    0x02
#define P54_LED_MODE_TXBLINK    0x03
#define P54_LED_MODE_ACTBLINK   0x04
#define P54_LED_MODE_PWM        0x05

#define P54_LED_A               0x01
#define P54_LED_B               0x02

struct p54_leds {
	int drv_auto;
	uint16_t mode;
	uint16_t perm_setting;
	uint16_t temp_setting;
	uint16_t duration;
};

/******************/
/*** P54PMALLOC ***/
/******************/

/* keep in sync with pmalloc.h */

#define P54_PMALLOC_MAX_BLOCKS  32

enum p54_pmalloc_purposes {
	P54_PMALLOC_PURPOSE_CONTROL = 0,
	P54_PMALLOC_PURPOSE_EEPROM = 1,
	P54_PMALLOC_PURPOSE_80211MGMT = 2,
	P54_PMALLOC_PURPOSE_80211DATA = 3,
	P54_PMALLOC_PURPOSE_80211BEACON = 4,
	P54_PMALLOC_PURPOSE_FWRESERVED = 5,
	P54_PMALLOC_PURPOSE_MISC = 6
};

struct p54_pmalloc_block {
	uint32_t start_address;
	uint32_t end_address;
	uint32_t headroom;
	int purpose;
};

struct p54_pmalloc {
	uint32_t start_address;
	uint32_t end_address;
	unsigned int allocated_blocks; /* number of allocated blocks */
	struct p54_pmalloc_block blocks[P54_PMALLOC_MAX_BLOCKS]; /* information about the allocated blocks, ordered by start address */
};

/****************/
/*** P54FRAME ***/
/****************/

/* keep in sync with P54U_MAX_DATA_FRAME_SIZE (usb_transport.h) */
#define P54_MAX_FRAME_SIZE 2048

struct p54_frame {
	uint32_t address;
	uint32_t size;
	char data[P54_MAX_FRAME_SIZE];
};

/******************/
/*** P54MEMREAD ***/
/******************/

struct p54_memread {
	uint32_t startaddr;
	uint32_t length;
	uint32_t *storage;
};

/********************/
/*** P54CRASHDUMP ***/
/********************/

struct p54_crashdump {
	uint32_t size;
	char *storage;
};

#endif

