/*
  Copyright (C) 2005, 2006 Sebastien Bourdeauducq <lekernel@prism54.org>
  Copyright (C) 2003 Intersil Americas Inc.
 
  This file is part of p54u.
 
  p54u is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  p54u is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with p54u ; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/param.h>
#include <termios.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

static int do_terminal(int upci_fd)
{
	char buff;
	struct termios otty, ntty;
	fd_set mask, rmask;
	int i;

	printf("[ TERMINAL. Use CTRL-] to leave ]\n");

	/* Set up the keyboard. */
	if(ioctl(0, TIOCGETA, &otty)  < 0) {
		perror("term ioctl TCGETS");
		return 0;
	}
	ntty = otty;
	ntty.c_lflag &= ~(ECHO|ICANON|ISIG);
	ntty.c_iflag &= ~(ICRNL|IXON|IXOFF);
	ntty.c_oflag &= ~(ONLCR|OPOST);
	if(ioctl(0, TIOCSETA, &ntty)  < 0) {
		perror("term ioctl TCSETS");
		return 0;
	}

	/* Set up the correct descriptor masks for select(2). */
	FD_ZERO(&mask);
	FD_SET(STDIN_FILENO, &mask);
	FD_SET(upci_fd, &mask);

	/* Go into an endless terminal loop. */
	while(1) {
		rmask = mask;
		i = select(32, &rmask, (fd_set *)NULL, (fd_set *)NULL, (struct timeval *)NULL);

		if(i<=0)
			break;

		if(FD_ISSET(STDIN_FILENO, &rmask)) {
			i = read(STDIN_FILENO, &buff, 1);
			if(i > 0) {
				if(buff == (']' & 037))
					break;
				write(upci_fd, &buff, i);
			}
		}

		if(FD_ISSET(upci_fd, &rmask)) {
			i = read(upci_fd, &buff, 1);
			if(i == -1) {
				perror("read");
				break;
			}
			if(i>0)
				write(STDOUT_FILENO, &buff, i);
		}
	}

	if(ioctl(0, TIOCSETA, &otty)  < 0) {
		perror("term ioctl TCSETS");
		return 0;
	}

	printf("\n[ Exiting ]\n");
	return 1;
}

int main(int argc, char *argv[])
{
	int fd;

	if(argc != 2) {
		fprintf(stderr, "Usage: p54term device\n");
		return 1;
	}
	fd = open(argv[1], O_RDWR);
	if(fd < 0) {
		perror("Unable to open device");
		return 1;
	}
	if(!do_terminal(fd)) {
		close(fd);
		return 1;
	}
	close(fd);
	return 0;
}
